/*
 * Copyright (c) 1983 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char sccsid[] = "@(#)dayobjs.c	5.3 (Berkeley) 6/1/90";
#endif /* not lint */

#include "externs.h"

struct objs dayobjs[] = {
	{ 236,	HORSE },
	{ 237,	CAR },
	{ 275,	POT },
	{ 275,	BAR },
	{ 275,	BLOCK },
	{ 260,	COINS },
	{ 266,	DARK },
	{ 235,	TIMER },
	{ 51,	51 },
	{ 59,	51 },
	{ 48,	51 },
	{ 66,	52 },
	{ 65,	52 },
	{ 19,	BOMB },
	{ 167,	NATIVE },
	{ 21,	KNIFE },
	{ 30,	KNIFE },
	{ 30,	CLEAVER },
	{ 260,	SWORD },
	{ 70,	LAND },
	{ 71,	LAND },
	{ 72,	LAND },
	{ 73,	LAND },
	{ 74,	LAND },
	{ 75,	LAND },
	{ 76,	LAND },
	{ 77,	LAND },
	{ 78,	LAND },
	{ 79,	LAND },
	{ 81,	LAND },
	{ 82,	LAND },
	{ 83,	LAND },
	{ 84,	LAND },
	{ 85,	LAND },
	{ 86,	LAND },
	{ 87,	LAND },
	{ 88,	LAND },
	{ 90,	LAND },
	{ 95,	LAND },
	{ 96,	LAND },
	{ 97,	LAND },
	{ 99,	LAND },
	{ 100,	LAND },
	{ 104,	LAND },
	{ 172,	WOODSMAN },
	{ 172,	DEADWOOD },
	{ 172,	MALLET },
	{ 146,	ELF },
	{ 146,	HALBERD },
	{ 146,	SHIELD },
	{ 190,	TWO_HANDED },
	{ 190,	POTION },
	{ 142,	BROAD },
	{ 258,	MAIL },
	{ 258,	HELM },
	{ 21,	MAID },
	{ 7,	VIPER },
	{ 216,	SHOES },
	{ 64,	CYLON },
	{ 36,	CYLON },
	{ 49,	CYLON },
	{ 8,	ROBE },
	{ 13,	AMULET },
	{ 20,	LASER },
	{ 126,	BATHGOD },
	{ 26,	GRENADE },
	{ 256,	GRENADE },
	{ 237,	CHAIN },
	{ 237,	COMPASS },
	{ 218,	LEVIS },
	{ 164,	MACE },
	{ 137,	SHOVEL },
	{ 11,	COINS },
	{ 24,	MATCHES },
	{ 235,	MATCHES },
	{ 93,	MAN },
	{ 109,	PAPAYAS },
	{ 110,	PINEAPPLE },
	{ 152,	PINEAPPLE },
	{ 154,	PINEAPPLE },
	{ 111,	KIWI },
	{ 149,	MANGO },
	{ 112,	COCONUTS },
	{ 150,	COCONUTS },
	{ 151,	COCONUTS },
	{ 153,	COCONUTS },
	{ 192,	COCONUTS },
	{ 204,	COCONUTS },
	{ 207,	COCONUTS },
	{ 209,	COCONUTS },
	{ 213,	COCONUTS },
	{ 240,	COCONUTS },
	{ 218,	RING },
	{ 130,	BRACELET },
	{ 93,	GIRL },
	{ 268,	LAMPON },
	0
};
